@@ -14,18 +14,21 @@
  * isn't installing.
  *
  * @since 2.3.0
- * @uses $wp_version Used to check against the newest WordPress version.
+ * @global string $wp_version Used to check against the newest WordPress version.
+ * @global wpdb   $wpdb
+ * @global string $wp_local_package
  *
  * @param array $extra_stats Extra statistics to report to the WordPress.org API.
- * @param bool $force_check Whether to bypass the transient cache and force a fresh update check. Defaults to false, true if $extra_stats is set.
- * @return mixed Returns null if update is unsupported. Returns false if check is too soon.
+ * @param bool  $force_check Whether to bypass the transient cache and force a fresh update check. Defaults to false, true if $extra_stats is set.
  */
 function wp_version_check( $extra_stats = array(), $force_check = false ) {
-	if ( defined('WP_INSTALLING') )
+	if ( wp_installing() ) {
 		return;
+	}
 
 	global $wpdb, $wp_local_package;
-	include( ABSPATH . WPINC . '/version.php' ); // include an unmodified $wp_version
+	// include an unmodified $wp_version
+	include( ABSPATH . WPINC . '/version.php' );
 	$php_version = phpversion();
 
 	$current = get_site_transient( 'update_core' );
@@ -47,10 +50,10 @@ function wp_version_check( $extra_stats = array(), $force_check = false ) {
 	// Wait 60 seconds between multiple version check requests
 	$timeout = 60;
 	$time_not_changed = isset( $current->last_checked ) && $timeout > ( time() - $current->last_checked );
-	if ( ! $force_check && $time_not_changed )
-		return false;
+	if ( ! $force_check && $time_not_changed ) {
+		return;
+	}
 
-	$locale = get_locale();
 	/**
 	 * Filter the locale requested for WordPress core translations.
 	 *
@@ -58,7 +61,7 @@ function wp_version_check( $extra_stats = array(), $force_check = false ) {
 	 *
 	 * @param string $locale Current locale.
 	 */
-	$locale = apply_filters( 'core_version_check_locale', $locale );
+	$locale = apply_filters( 'core_version_check_locale', get_locale() );
 
 	// Update last_checked for current to prevent multiple blocking requests if request hangs
 	$current->last_checked = time();
@@ -83,18 +86,19 @@ function wp_version_check( $extra_stats = array(), $force_check = false ) {
 	}
 
 	$query = array(
-		'version'           => $wp_version,
-		'php'               => $php_version,
-		'locale'            => $locale,
-		'mysql'             => $mysql_version,
-		'local_package'     => isset( $wp_local_package ) ? $wp_local_package : '',
-		'blogs'             => $num_blogs,
-		'users'             => $user_count,
-		'multisite_enabled' => $multisite_enabled,
+		'version'            => $wp_version,
+		'php'                => $php_version,
+		'locale'             => $locale,
+		'mysql'              => $mysql_version,
+		'local_package'      => isset( $wp_local_package ) ? $wp_local_package : '',
+		'blogs'              => $num_blogs,
+		'users'              => $user_count,
+		'multisite_enabled'  => $multisite_enabled,
+		'initial_db_version' => get_site_option( 'initial_db_version' ),
 	);
 
 	$post_body = array(
-		'translations' => json_encode( $translations ),
+		'translations' => wp_json_encode( $translations ),
 	);
 
 	if ( is_array( $extra_stats ) )
@@ -120,14 +124,16 @@ function wp_version_check( $extra_stats = array(), $force_check = false ) {
 		$response = wp_remote_post( $http_url, $options );
 	}
 
-	if ( is_wp_error( $response ) || 200 != wp_remote_retrieve_response_code( $response ) )
-		return false;
+	if ( is_wp_error( $response ) || 200 != wp_remote_retrieve_response_code( $response ) ) {
+		return;
+	}
 
 	$body = trim( wp_remote_retrieve_body( $response ) );
 	$body = json_decode( $body, true );
 
-	if ( ! is_array( $body ) || ! isset( $body['offers'] ) )
-		return false;
+	if ( ! is_array( $body ) || ! isset( $body['offers'] ) ) {
+		return;
+	}
 
 	$offers = $body['offers'];
 
@@ -142,7 +148,7 @@ function wp_version_check( $extra_stats = array(), $force_check = false ) {
 				$offer[ $offer_key ] = esc_html( $value );
 		}
 		$offer = (object) array_intersect_key( $offer, array_fill_keys( array( 'response', 'download', 'locale',
-			'packages', 'current', 'version', 'php_version', 'mysql_version', 'new_bundled', 'partial_version', 'notify_email', 'support_email' ), '' ) );
+			'packages', 'current', 'version', 'php_version', 'mysql_version', 'new_bundled', 'partial_version', 'notify_email', 'support_email', 'new_files' ), '' ) );
 	}
 
 	$updates = new stdClass();
@@ -163,7 +169,7 @@ function wp_version_check( $extra_stats = array(), $force_check = false ) {
 		}
 	}
 
-	// Trigger a background updates check if running non-interactively, and we weren't called from the update handler.
+	// Trigger background updates if running non-interactively, and we weren't called from the update handler.
 	if ( defined( 'DOING_CRON' ) && DOING_CRON && ! doing_action( 'wp_maybe_auto_update' ) ) {
 		do_action( 'wp_maybe_auto_update' );
 	}
@@ -177,16 +183,17 @@ function wp_version_check( $extra_stats = array(), $force_check = false ) {
  * api.wordpress.org. Will only check if WordPress isn't installing.
  *
  * @since 2.3.0
- * @uses $wp_version Used to notify the WordPress version.
+ * @global string $wp_version Used to notify the WordPress version.
  *
  * @param array $extra_stats Extra statistics to report to the WordPress.org API.
- * @return mixed Returns null if update is unsupported. Returns false if check is too soon.
  */
 function wp_update_plugins( $extra_stats = array() ) {
-	include( ABSPATH . WPINC . '/version.php' ); // include an unmodified $wp_version
+	if ( wp_installing() ) {
+		return;
+	}
 
-	if ( defined('WP_INSTALLING') )
-		return false;
+	// include an unmodified $wp_version
+	include( ABSPATH . WPINC . '/version.php' );
 
 	// If running blog-side, bail unless we've not checked in the last 12 hours
 	if ( !function_exists( 'get_plugins' ) )
@@ -244,8 +251,9 @@ function wp_update_plugins( $extra_stats = array() ) {
 		}
 
 		// Bail if we've checked recently and if nothing has changed
-		if ( ! $plugin_changed )
-			return false;
+		if ( ! $plugin_changed ) {
+			return;
+		}
 	}
 
 	// Update last_checked for current to prevent multiple blocking requests if request hangs
@@ -254,15 +262,18 @@ function wp_update_plugins( $extra_stats = array() ) {
 
 	$to_send = compact( 'plugins', 'active' );
 
-	$locales = array( get_locale() );
+	$locales = array_values( get_available_languages() );
+
 	/**
 	 * Filter the locales requested for plugin translations.
 	 *
 	 * @since 3.7.0
+	 * @since 4.5.0 The default value of the `$locales` parameter changed to include all locales.
 	 *
-	 * @param array $locales Plugin locale. Default is current locale of the site.
+	 * @param array $locales Plugin locales. Default is all available locales of the site.
 	 */
 	$locales = apply_filters( 'plugins_update_check_locales', $locales );
+	$locales = array_unique( $locales );
 
 	if ( defined( 'DOING_CRON' ) && DOING_CRON ) {
 		$timeout = 30;
@@ -274,16 +285,16 @@ function wp_update_plugins( $extra_stats = array() ) {
 	$options = array(
 		'timeout' => $timeout,
 		'body' => array(
-			'plugins'      => json_encode( $to_send ),
-			'translations' => json_encode( $translations ),
-			'locale'       => json_encode( $locales ),
-			'all'          => json_encode( true ),
+			'plugins'      => wp_json_encode( $to_send ),
+			'translations' => wp_json_encode( $translations ),
+			'locale'       => wp_json_encode( $locales ),
+			'all'          => wp_json_encode( true ),
 		),
 		'user-agent' => 'WordPress/' . $wp_version . '; ' . get_bloginfo( 'url' )
 	);
 
 	if ( $extra_stats ) {
-		$options['body']['update_stats'] = json_encode( $extra_stats );
+		$options['body']['update_stats'] = wp_json_encode( $extra_stats );
 	}
 
 	$url = $http_url = 'http://api.wordpress.org/plugins/update-check/1.1/';
@@ -296,14 +307,21 @@ function wp_update_plugins( $extra_stats = array() ) {
 		$raw_response = wp_remote_post( $http_url, $options );
 	}
 
-	if ( is_wp_error( $raw_response ) || 200 != wp_remote_retrieve_response_code( $raw_response ) )
-		return false;
+	if ( is_wp_error( $raw_response ) || 200 != wp_remote_retrieve_response_code( $raw_response ) ) {
+		return;
+	}
 
 	$response = json_decode( wp_remote_retrieve_body( $raw_response ), true );
 	foreach ( $response['plugins'] as &$plugin ) {
 		$plugin = (object) $plugin;
+		if ( isset( $plugin->compatibility ) ) {
+			$plugin->compatibility = (object) $plugin->compatibility;
+			foreach ( $plugin->compatibility as &$data ) {
+				$data = (object) $data;
+			}
+		}
 	}
-	unset( $plugin );
+	unset( $plugin, $data );
 	foreach ( $response['no_update'] as &$plugin ) {
 		$plugin = (object) $plugin;
 	}
@@ -331,16 +349,16 @@ function wp_update_plugins( $extra_stats = array() ) {
  * installing.
  *
  * @since 2.7.0
- * @uses $wp_version Used to notify the WordPress version.
  *
  * @param array $extra_stats Extra statistics to report to the WordPress.org API.
- * @return mixed Returns null if update is unsupported. Returns false if check is too soon.
  */
 function wp_update_themes( $extra_stats = array() ) {
-	include( ABSPATH . WPINC . '/version.php' ); // include an unmodified $wp_version
+	if ( wp_installing() ) {
+		return;
+	}
 
-	if ( defined( 'WP_INSTALLING' ) )
-		return false;
+	// include an unmodified $wp_version
+	include( ABSPATH . WPINC . '/version.php' );
 
 	$installed_themes = wp_get_themes();
 	$translations = wp_get_installed_translations( 'themes' );
@@ -407,8 +425,9 @@ function wp_update_themes( $extra_stats = array() ) {
 		}
 
 		// Bail if we've checked recently and if nothing has changed
-		if ( ! $theme_changed )
-			return false;
+		if ( ! $theme_changed ) {
+			return;
+		}
 	}
 
 	// Update last_checked for current to prevent multiple blocking requests if request hangs
@@ -417,15 +436,18 @@ function wp_update_themes( $extra_stats = array() ) {
 
 	$request['themes'] = $themes;
 
-	$locales = array( get_locale() );
+	$locales = array_values( get_available_languages() );
+
 	/**
 	 * Filter the locales requested for theme translations.
 	 *
 	 * @since 3.7.0
+	 * @since 4.5.0 The default value of the `$locales` parameter changed to include all locales.
 	 *
-	 * @param array $locales Theme locale. Default is current locale of the site.
+	 * @param array $locales Theme locales. Default is all available locales of the site.
 	 */
 	$locales = apply_filters( 'themes_update_check_locales', $locales );
+	$locales = array_unique( $locales );
 
 	if ( defined( 'DOING_CRON' ) && DOING_CRON ) {
 		$timeout = 30;
@@ -437,15 +459,15 @@ function wp_update_themes( $extra_stats = array() ) {
 	$options = array(
 		'timeout' => $timeout,
 		'body' => array(
-			'themes'       => json_encode( $request ),
-			'translations' => json_encode( $translations ),
-			'locale'       => json_encode( $locales ),
+			'themes'       => wp_json_encode( $request ),
+			'translations' => wp_json_encode( $translations ),
+			'locale'       => wp_json_encode( $locales ),
 		),
 		'user-agent'	=> 'WordPress/' . $wp_version . '; ' . get_bloginfo( 'url' )
 	);
 
 	if ( $extra_stats ) {
-		$options['body']['update_stats'] = json_encode( $extra_stats );
+		$options['body']['update_stats'] = wp_json_encode( $extra_stats );
 	}
 
 	$url = $http_url = 'http://api.wordpress.org/themes/update-check/1.1/';
@@ -458,8 +480,9 @@ function wp_update_themes( $extra_stats = array() ) {
 		$raw_response = wp_remote_post( $http_url, $options );
 	}
 
-	if ( is_wp_error( $raw_response ) || 200 != wp_remote_retrieve_response_code( $raw_response ) )
-		return false;
+	if ( is_wp_error( $raw_response ) || 200 != wp_remote_retrieve_response_code( $raw_response ) ) {
+		return;
+	}
 
 	$new_update = new stdClass;
 	$new_update->last_checked = time();
@@ -492,12 +515,13 @@ function wp_maybe_auto_update() {
  * Retrieves a list of all language updates available.
  *
  * @since 3.7.0
+ *
+ * @return array
  */
 function wp_get_translation_updates() {
 	$updates = array();
 	$transients = array( 'update_core' => 'core', 'update_plugins' => 'plugin', 'update_themes' => 'theme' );
 	foreach ( $transients as $transient => $type ) {
-
 		$transient = get_site_transient( $transient );
 		if ( empty( $transient->translations ) )
 			continue;
@@ -506,7 +530,6 @@ function wp_get_translation_updates() {
 			$updates[] = (object) $translation;
 		}
 	}
-
 	return $updates;
 }
 
@@ -571,17 +594,24 @@ function wp_get_update_data() {
 	return apply_filters( 'wp_get_update_data', $update_data, $titles );
 }
 
+/**
+ * Determines whether core should be updated.
+ *
+ * @since 2.8.0
+ *
+ * @global string $wp_version
+ */
 function _maybe_update_core() {
-	include( ABSPATH . WPINC . '/version.php' ); // include an unmodified $wp_version
+	// include an unmodified $wp_version
+	include( ABSPATH . WPINC . '/version.php' );
 
 	$current = get_site_transient( 'update_core' );
 
-	if ( isset( $current->last_checked ) &&
+	if ( isset( $current->last_checked, $current->version_checked ) &&
 		12 * HOUR_IN_SECONDS > ( time() - $current->last_checked ) &&
-		isset( $current->version_checked ) &&
-		$current->version_checked == $wp_version )
+		$current->version_checked == $wp_version ) {
 		return;
-
+	}
 	wp_version_check();
 }
 /**
@@ -614,7 +644,6 @@ function _maybe_update_themes() {
 	$current = get_site_transient( 'update_themes' );
 	if ( isset( $current->last_checked ) && 12 * HOUR_IN_SECONDS > ( time() - $current->last_checked ) )
 		return;
-
 	wp_update_themes();
 }
 
@@ -624,21 +653,34 @@ function _maybe_update_themes() {
  * @since 3.1.0
  */
 function wp_schedule_update_checks() {
-	if ( !wp_next_scheduled('wp_version_check') && !defined('WP_INSTALLING') )
+	if ( ! wp_next_scheduled( 'wp_version_check' ) && ! wp_installing() )
 		wp_schedule_event(time(), 'twicedaily', 'wp_version_check');
 
-	if ( !wp_next_scheduled('wp_update_plugins') && !defined('WP_INSTALLING') )
+	if ( ! wp_next_scheduled( 'wp_update_plugins' ) && ! wp_installing() )
 		wp_schedule_event(time(), 'twicedaily', 'wp_update_plugins');
 
-	if ( !wp_next_scheduled('wp_update_themes') && !defined('WP_INSTALLING') )
+	if ( ! wp_next_scheduled( 'wp_update_themes' ) && ! wp_installing() )
 		wp_schedule_event(time(), 'twicedaily', 'wp_update_themes');
-
-	if ( ( wp_next_scheduled( 'wp_maybe_auto_update' ) > ( time() + HOUR_IN_SECONDS ) ) && ! defined('WP_INSTALLING') )
-		wp_clear_scheduled_hook( 'wp_maybe_auto_update' );
 }
 
-if ( ( ! is_main_site() && ! is_network_admin() ) || ( defined( 'DOING_AJAX' ) && DOING_AJAX ) )
+/**
+ * Clear existing update caches for plugins, themes, and core.
+ *
+ * @since 4.1.0
+ */
+function wp_clean_update_cache() {
+	if ( function_exists( 'wp_clean_plugins_cache' ) ) {
+		wp_clean_plugins_cache();
+	} else {
+		delete_site_transient( 'update_plugins' );
+	}
+	wp_clean_themes_cache();
+	delete_site_transient( 'update_core' );
+}
+
+if ( ( ! is_main_site() && ! is_network_admin() ) || ( defined( 'DOING_AJAX' ) && DOING_AJAX ) ) {
 	return;
+}
 
 add_action( 'admin_init', '_maybe_update_core' );
 add_action( 'wp_version_check', 'wp_version_check' );
@@ -658,6 +700,8 @@ add_action( 'admin_init', '_maybe_update_themes' );
 add_action( 'wp_update_themes', 'wp_update_themes' );
 add_action( 'upgrader_process_complete', 'wp_update_themes', 10, 0 );
 
+add_action( 'update_option_WPLANG', 'wp_clean_update_cache' , 10, 0 );
+
 add_action( 'wp_maybe_auto_update', 'wp_maybe_auto_update' );
 
-add_action('init', 'wp_schedule_update_checks');
+add_action( 'init', 'wp_schedule_update_checks' );
